//=============================================================================
// AK47Weapon
//=============================================================================

class AK47Weapon extends CatableWeapon;

#exec OBJ LOAD FILE=TnMesh.usx
#exec OBJ LOAD FILE=TnSound.uax
#exec OBJ LOAD FILE=TnWeapons.ukx

///////////////////////////////////////////////////////////////////////////////
// Vars, structs, consts
///////////////////////////////////////////////////////////////////////////////
var travel byte ShootScissors;		// Cheat that makes you shoot bouncing scissors instead
var float SPAccuracy;				// How accurate we are in single player games.

var AK47 TAK;

///////////////////////////////////////////////////////////////////////////////
simulated function PostBeginPlay()
{
	local float diffoffset;

	Super.PostBeginPlay();

	// If it's a single player game, make our accuracy much worse
	if(Level.Game != None
		&& FPSGameInfo(Level.Game).bIsSinglePlayer)
	{
		TraceAccuracy = SPAccuracy;

		// Based on the difficulty, bump up the number of times AI will continuously
		// shoot this weapon
		diffoffset = P2GameInfo(Level.Game).GetDifficultyOffset();
		if(diffoffset > 0)
		{
			AI_BurstCountExtra+=(diffoffset/2);
		}
	}
}

simulated function PostNetBeginPlay()
{
    local PlayerController PC;
	local Pawn P;

    Super.PostNetBeginPlay();

    P  = Pawn(Owner);
	PC = PlayerController(P.Controller);

   	if (P.IsLocallyControlled() && PC!=None && (!PC.bBehindView) )
	{
        TAK = Spawn(Class'TnInventory.AK47');
        AttachToBone( TAK, 'MP5' );
    }
   	else
	{
        if(Instigator != None)
	   {
		 if(Instigator.Mesh != None)

             TAK.Destroyed();
             TAK = None;
	   }
	}

}

///////////////////////////////////////////////////////////////////////////////

function DropFrom(vector StartLocation)
{

   Super.DropFrom(StartLocation);

     TAK.Destroyed();

}

simulated function Destroyed()
{
   Super.Destroyed();

  if( TAK != None )
  {
    DetachFromBone(TAK);
    TAK.Destroy();
    TAK = None;
  }
}

///////////////////////////////////////////////////////////////////////////////
// This will randomly change the color and the size of the dynamic
// light associate with the flash. Change in each weapon's file,
// but call each time you start up the flash again.
// This function is also used by the third-person muzzle flash, so the 
// colors will look the same
///////////////////////////////////////////////////////////////////////////////
simulated function PickLightValues()
{
	LightBrightness=150;
	LightSaturation=180;
	LightHue=12+FRand()*15;
	LightRadius=16+FRand()*16;
}

///////////////////////////////////////////////////////////////////////////////
// Tell ammo to make this scissors type
///////////////////////////////////////////////////////////////////////////////
function SpawnScissors(bool bMakeSpinner)
{
	local vector HitLocation, HitNormal, StartTrace, EndTrace, X,Y,Z;
	local actor Other;
	local ScissorsProjectile sic;
	local P2Player p2p;
	
	GetAxes(Instigator.GetViewRotation(),X,Y,Z);
	StartTrace = GetFireStart(X,Y,Z); 
	AdjustedAim = Instigator.AdjustAim(AmmoType, StartTrace, 2*AimError);	
	sic = spawn(class'ScissorsAltProjectile',Instigator,,StartTrace, AdjustedAim);
}

///////////////////////////////////////////////////////////////////////////////
// See what we hit
///////////////////////////////////////////////////////////////////////////////
function TraceFire( float Accuracy, float YOffset, float ZOffset )
{
	local int i;
	
	// Reduce the ammo only by 1 here, for the shotgun, but shoot
	// ShotCountMaxForNotify number of pellets each time.
	P2AmmoInv(AmmoType).UseAmmoForShot();

	// Reduce the cat ammo if we're using one
	if(CatOnGun == 1)
		CatAmmoLeft--;

	if(ShootScissors==1)
		SpawnScissors(true);
	else
		Super.TraceFire(Accuracy, YOffset, ZOffset);
}

///////////////////////////////////////////////////////////////////////////////
// DrawMuzzleFlash()
//Default implementation assumes that flash texture can be drawn inverted in X and/or Y direction to increase apparent
//number of flash variations
// We might need to draw a dark blood splat
///////////////////////////////////////////////////////////////////////////////
simulated function DrawMuzzleFlash(Canvas Canvas)
{
	local float Scale, ULength, VLength, UStart, VStart;

	// Form dynamic muzzle flash size
	MuzzleScale = FRand()*(default.MuzzleScale);
	// Form offset based on that
//	FlashOffsetX = -(MuzzleScale/500) + default.FlashOffsetX;
//	FlashOffsetY = -(MuzzleScale/500) + default.FlashOffsetY;

	MuzzleScale += 1.0;

	if(CatOnGun == 0
		|| !class'P2Player'.static.BloodMode())
		Super.DrawMuzzleFlash(Canvas);
	else	// if a cat, then draw it alpha blended with a gross blood splat
	{
		Scale = MuzzleScale * Canvas.ClipX/640.0;
		Canvas.Style = ERenderStyle.STY_Alpha;
		ULength = MFTexture.USize;
		if ( FRand() <= 0.5 )
		{
			UStart = ULength;
			ULength = -1 * ULength;
		}
		VLength = MFTexture.VSize;
		if ( FRand() <= 0.5 )
		{
			VStart = VLength;
			VLength = -1 * VLength;
		}

		Canvas.DrawTile(MFTexture, Scale * MFTexture.USize, Scale * MFTexture.VSize, 
					UStart, VStart, ULength, VLength);
		Canvas.Style = ERenderStyle.STY_Normal;
	}
}
state ClientFiring
{
}
state NormalFire
{
}

///////////////////////////////////////////////////////////////////////////////
// Default properties
///////////////////////////////////////////////////////////////////////////////

defaultproperties
{
	 SPAccuracy=0.700000
     CatMesh=SkeletalMesh'TnWeapons.Tn_Dude_MachinegunCat'
     MFBloodTexture=Texture'nathans.muzzleflashes.bloodmuzzleflash'
     StartShotsWithCat=9
     CatFireSound=Sound'WeaponSounds.machinegun_catfire'
     ViolenceRank=5
     RecognitionDist=1300.000000
     ShotCountMaxForNotify=20
     AI_BurstCountExtra=10
     AI_BurstCountMin=6
     AI_BurstTime=0.150000
     holdstyle=WEAPONHOLDSTYLE_Both
     switchstyle=WEAPONHOLDSTYLE_Both
     firingstyle=WEAPONHOLDSTYLE_Both
     MinRange=300.000000
     ShakeOffsetTime=2.000000
     CombatRating=5.000000
     FirstPersonMeshSuffix="MachineGun"
     WeaponSpeedLoad=1.500000
     WeaponSpeedReload=1.500000
     WeaponSpeedHolster=1.500000
     WeaponSpeedShoot1=25.000000
     AmmoName=Class'TnInventory.AK47AmmoInv'
     AutoSwitchPriority=4
     ShakeRotMag=(X=120.000000,Y=30.000000,Z=30.000000)
     ShakeOffsetMag=(X=3.000000,Y=2.500000,Z=2.500000)
     TraceAccuracy=0.110000
     aimerror=600.000000
     AIRating=0.400000
     MaxRange=1200.000000
     FireSound=Sound'TnSound.Shot_AK47'
     MuzzleScale=0.200000
     FlashOffsetY=-0.030000
     FlashOffsetX=0.030000
     MuzzleFlashSize=128.000000
     MFTexture=Texture'Timb.muzzle_flash.machine_gun_corona'
     bDrawMuzzleFlash=True
     MuzzleFlashScale=2.400000
     MuzzleFlashStyle=STY_Normal
     InventoryGroup=4
     GroupOffset=3
     PickupClass=Class'TnInventory.AK47Pickup'
     PlayerViewOffset=(X=1.000000,Y=0.300000,Z=-1.300000)
     BobDamping=0.975000
     AttachmentClass=Class'TnInventory.AK47Attachment'
	 ItemName="AK47"
     Mesh=SkeletalMesh'TnWeapons.Tn_Dude_Machinegun'
     Skins(0)=Texture'WeaponSkins.Dude_Hands'
     Skins(1)=Texture'None'
     Skins(2)=Texture'None'
     Skins(3)=Texture'AnimalSkins.Cat_Orange'
     SoundRadius=255.000000
	 SelectSound=Sound'TnSound.Select_AK47'
}
