///////////////////////////////////////////////////////////////////////////////
// M82A1Projectile.
///////////////////////////////////////////////////////////////////////////////
class M82A1Projectile extends P2Projectile;


const FLESH_HIT_MAX	=	4;
const WAIT_TIME = 0.05;
const MIN_SIZE = 10.0;

var class<TimedMarker> PawnHitMarkerMade;	// danger made when bullet hits a pawn 
var class<TimedMarker> BulletHitMarkerMade;	// danger made when bullet hits dirt
var Sound FleshHit[FLESH_HIT_MAX];
var float FleshRad;
var vector Direction;

///////////////////////////////////////////////////////////////////////////////
// Send it off in the right direction
///////////////////////////////////////////////////////////////////////////////
function SetVelocity(vector Dir)
{
	Direction = Dir;
	Velocity = Speed*Direction;
}


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Flying through the air
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
auto state Moving
{
	ignores HitWall, Explode, Blowup, ProcessTouch, Touch, EncroachingOn;

	///////////////////////////////////////////////////////////////////////////////
	// Make people notice a body dropping limply before them from a sniper hit
	///////////////////////////////////////////////////////////////////////////////
	function TellNPCs(Actor Other, vector HitLocation)
	{
		if(Pawn(Other) != None)
		{
			// Set your enemy as the one you attacked.
			if(P2Player(Instigator.Controller) != None
				&& FPSPawn(Other) != None)
			{
				P2Player(Instigator.Controller).Enemy = FPSPawn(Other);
			}

			PawnHitMarkerMade.static.NotifyControllersStatic(
				Level,
				PawnHitMarkerMade,
				FPSPawn(Owner), 
				FPSPawn(Other), 
				PawnHitMarkerMade.default.CollisionRadius,
				HitLocation);
		}
		else	// wall hit.. still have them notice
		{
			BulletHitMarkerMade.static.NotifyControllersStatic(
				Level,
				BulletHitMarkerMade,
				FPSPawn(Owner), 
				None, 
				BulletHitMarkerMade.default.CollisionRadius,
				HitLocation);
		}
	}

	///////////////////////////////////////////////////////////////////////////////
	// False means you hit something you can't go past so die.
	///////////////////////////////////////////////////////////////////////////////
	function bool ProcessTraceHit(Actor Other, out vector HitLocation, Vector HitNormal)
	{
		local Rotator NewRot;
		local float usesize;

		if ( Other == None )
		{
			return true;
		}

//		if(P2GameInfoSingle(Level.Game) != None
//			&& P2GameInfoSingle(Level.Game).VerifySeqTime()
//			&& P2Pawn(Instigator) != None
//			&& P2Pawn(Instigator).bPlayer)
//		{
//			GenExplosion(HitLocation, HitNormal, Other);
//			return false;
//		}

		TellNPCs(Other, HitLocation);

		if(Other.bStatic)
		{
//			spawn(class'M82A1HitPack',Owner, ,HitLocation, Rotator(HitNormal));
			spawn(class'C4Explosion',Owner, ,HitLocation, Rotator(HitNormal));
			
			return false;
		}
		else 
		{
			Other.TakeDamage(class'M82A1AmmoInv'.default.DamageAmount, Pawn(Owner), 
							HitLocation, Velocity, class'M82A1AmmoInv'.default.DamageTypeInflicted);

			// If we're a guy (krotchy) who doesn't take head shots at all, stop the bullet completely)
			if(P2Pawn(Other) != None
				&& P2Pawn(Other).TakesRifleHeadShot == 0)
			{
				spawn(class'BulletSparkPack',Owner, ,HitLocation, Rotator(HitNormal));
				return false;
			}
			else if((Pawn(Other) != None 
					&& Other != Owner)
				|| PeoplePart(Other) != None
				|| CowheadProjectile(Other) != None)
			{
				Other.PlaySound(FleshHit[Rand(ArrayCount(FleshHit))],SLOT_Pain,,,FleshRad,GetRandPitch());
			}
			else if(Other.bBlockActors)
			{
				spawn(class'BulletSparkPack',Owner, ,HitLocation, Rotator(HitNormal));
			}
			// Move to the other side of the thing you hit
			usesize = Other.CollisionRadius + Other.CollisionHeight;
			if(usesize < MIN_SIZE)
				usesize = MIN_SIZE;
			HitLocation += Direction*(usesize);
		}
		return true;
	}
	///////////////////////////////////////////////////////////////////////////////
	// Moves along next path and checks everything for hit damage. Goes through
	// pawns and windows, etc, till it hits static things.
	///////////////////////////////////////////////////////////////////////////////
	function TraceFire()
	{
		local vector HitLocation, HitNormal, StartTrace, EndTrace;
		local actor Other;
		
		//if(!SetLocation(OldLocation))
		//	Destroy();
		// Do collision test
		StartTrace = Location;
		EndTrace = StartTrace + Velocity*WAIT_TIME;

		Other = Trace(HitLocation,HitNormal,EndTrace,StartTrace,true);

		if ( Other == None )
			HitLocation = EndTrace;

		if(ProcessTraceHit(Other, HitLocation, HitNormal))
		{
			if(!SetLocation(HitLocation))
				Destroy();
			//else
			//{
			//	OldLocation = HitLocation;
			//}
		}
		else
			Destroy();

	}
	
	//function BeginState()
	//{
	//	OldLocation = Location;
	//}

Begin:
	TraceFire();
	Sleep(WAIT_TIME);
	Goto('Begin');
}

defaultproperties
{
     PawnHitMarkerMade=Class'Postal2Game.PawnShotMarker'
     BulletHitMarkerMade=Class'Postal2Game.BulletHitMarker'
     FleshRad=200.000000
     speed=20000.000000
     MaxSpeed=20000.000000
     MomentumTransfer=50000.000000
     MyDamageType=Class'BaseFX.RifleDamage'
     Physics=PHYS_None
     DrawType=DT_StaticMesh
     LifeSpan=10.000000
     TransientSoundRadius=400.000000
     bCollideActors=False
     bCollideWorld=False
     bBlockZeroExtentTraces=False
     bBlockNonZeroExtentTraces=False
}
