///////////////////////////////////////////////////////////////////////////////
// MeatPackageProjectile.
///////////////////////////////////////////////////////////////////////////////
class MeatPackageProjectile extends CowHeadProjectile;

///////////////////////////////////////////////////////////////////////////////
function TakeDamage( int Dam, Pawn instigatedBy, Vector hitlocation, 
						Vector momentum, class<DamageType> damageType)
{
	local CatExplosion exp;
	local Rotator NewRot;

	if(ClassIsChildOf(damageType, class'BurnedDamage'))
	{
		// mark it as burned and swap out the fire texture
		if(!bBurned)
		{
			bBurned = true;
			Skins[0] = BurnSkin;
			if(clingball != None)
			{
				clingball.GotoState('WaitAndFade');
				clingball = None;
			}
		}
		return;
	}

	if(!ClassIsChildOf(damageType, class'AnthDamage')
		&& damageType != class'ElectricalDamage'
		&& Dam > 0)
	{
		// Don't move it if it's hit by a bullet, so you can shoot it right
		// where you dropped it.
		if(!ClassIsChildOf(damageType, class'BulletDamage')
			&& damageType != class'CuttingDamage')
		{
			bBounce=true;
			Acceleration = default.Acceleration;
			Velocity += momentum/Mass;
		}

		if(InstigatedBy != None
			&& !ClassIsChildOf(damageType, class'BludgeonDamage'))
		{
			// If you don't already have a big cloud around you, 
			// make one after being hurt
			MakeAnthCloud(vect(0,0,1.0));
		}
		// Someone must inflict the damage before we care. When we fall, we get hurt
		// by SmashDamage and instigated by no one--we don't want that to reset this.
		else if(InstigatedBy != None)
		{
			bMakeCloudOnBounce=true;
		}

		// Remove life
		Health -= Dam;

		if(Health <= 0)
		{
			if(class'P2Player'.static.BloodMode())
			{
				// Make blood explosion
				exp = spawn(class'CatExplosion',,,HitLocation);
				exp.PlaySound(ExplodeSound,,,,,GetRandPitch());
			}

 			Destroy();
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
defaultproperties
{
	 Health=80
     CowHeadBounce=Sound'MiscSounds.PickupSounds.PickupBounce'
     ExplodeSound=Sound'WeaponSounds.flesh_explode'
     BurnSkin=Texture'ben_Skins.Misc.meat_ben'
     StaticMesh=StaticMesh'ben_mesh.Misc.meat_packaged_ben'
	 Skins(0)=Texture'ben_Skins.Misc.meat_ben'
     AmbientGlow=64
}
