///////////////////////////////////////////////////////////////////////////////
// NapalmLauncher Weapon
///////////////////////////////////////////////////////////////////////////////

class NapalmLauncherWeapon extends P2Weapon;

///////////////////////////////////////////////////////////////////////////////
// Vars, structs, consts
///////////////////////////////////////////////////////////////////////////////
var travel bool bShowHint1;
var bool bAltFired;
var NapalmLauncher TNA;

simulated function PostNetBeginPlay()
{
    local PlayerController PC;
	local Pawn P;

    Super.PostNetBeginPlay();

    P  = Pawn(Owner);
	PC = PlayerController(P.Controller);

   	if (P.IsLocallyControlled() && PC!=None && (!PC.bBehindView) )
	{
        TNA = Spawn(Class'TnInventory.NapalmLauncher');
        AttachToBone( TNA, 'Napalm' );
    }
   	else
	{
        if(Instigator != None)
	   {
		 if(Instigator.Mesh != None)

             TNA.Destroyed();
             TNA = None;
	   }
	}

}


function DropFrom(vector StartLocation)
{

   Super.DropFrom(StartLocation);

     TNA.Destroyed();

}

simulated function Destroyed()
{
   Super.Destroyed();

  if( TNA != None )
  {
    DetachFromBone(TNA);
    TNA.Destroy();
    TNA = None;
  }
}
///////////////////////////////////////////////////////////////////////////////
// Give hints about this item
///////////////////////////////////////////////////////////////////////////////
function bool GetHints(out String str1, out String str2, out String str3,
				out byte InfiniteHintTime)
{
	if(bAllowHints)
	{
		if(bShowHint1)
			str1=HudHint1;
		else
			str2=HudHint2;
		return true;
	}
	return false;
}
///////////////////////////////////////////////////////////////////////////////
// Allow hints again
///////////////////////////////////////////////////////////////////////////////
function RefreshHints()
{
	Super.RefreshHints();
	bShowHint1=true;
}

simulated function PlayFiring()
{
	bForceReload=true;
	Super.PlayFiring();
	if(bShowHint1)
	{
		bShowHint1=false;
		UpdateHudHints();
	}
}
simulated function PlayAltFiring()
{
	bForceReload=true;
	Super.PlayAltFiring();
	if(!bShowHint1)
		TurnOffHint();
}

///////////////////////////////////////////////////////////////////////////////
// play reloading sounds
///////////////////////////////////////////////////////////////////////////////
simulated function PlayReloading()
{
	// temp speed up because with no animation, you can't tell you're
	// why you can't shoot (becuase it's playing a reload anim)
	PlayAnim('Shoot1Unload', WeaponSpeedReload, 0.05);
	Instigator.PlayOwnedSound(ReloadSound, SLOT_Misc, 1.0);
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
simulated function PlayDownAnim()
{
	if(HasAmmo())
		Super.PlayDownAnim();
	else
		PlayAnim('HolsterEmpty', WeaponSpeedHolster, 0.05);
		Instigator.PlayOwnedSound(HolsterSound, SLOT_Misc, 1.0);
}

///////////////////////////////////////////////////////////////////////////////
// Normal projectile fire, plus check where to make the danger marker to 
// alert people of the noise.
///////////////////////////////////////////////////////////////////////////////
function ProjectileFire()
{
	// STUB
}
///////////////////////////////////////////////////////////////////////////////
// Alt projectile fire, plus check where to make the danger marker to 
// alert people of the noise.
///////////////////////////////////////////////////////////////////////////////
function ProjectileAltFire()
{
	// STUB
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function Notify_ShootNapalm()
{
	local vector HitLocation, HitNormal, StartTrace, EndTrace, X,Y,Z;
	local actor HitActor;
	local NapalmProjectile npro;
	
	if(AmmoType != None
		&& AmmoType.HasAmmo())
	{
		GetAxes(Instigator.GetViewRotation(),X,Y,Z);
		StartTrace = GetFireStart(X,Y,Z); 
		AdjustedAim = Instigator.AdjustAim(AmmoType, StartTrace, 2*AimError);	
		//  Make sure there's nothing right in the way
		HitActor = Trace(HitLocation, HitNormal, Instigator.Location, StartTrace, true);
		if(HitActor == None
			|| (!HitActor.bStatic
				&& !HitActor.bWorldGeometry))
		{
			npro = spawn(class'NapalmProjectile',Instigator,,StartTrace, AdjustedAim);
			if(npro != None)
			{
				P2AmmoInv(AmmoType).UseAmmoForShot();
				npro.Instigator = Instigator;
				npro.SetupShot(!bAltFiring, !bAltFiring);
				npro.AddRelativeVelocity(Instigator.Velocity);
			}
			// Touch any actor that was in between, just in case.
			if(HitActor != None)
				HitActor.Bump(npro);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// Default properties
///////////////////////////////////////////////////////////////////////////////

defaultproperties
{
     bShowHint1=True
     bUsesAltFire=True
     ViolenceRank=8
     RecognitionDist=1500.000000
     ShotCountMaxForNotify=0
     holdstyle=WEAPONHOLDSTYLE_Both
     switchstyle=WEAPONHOLDSTYLE_Both
     firingstyle=WEAPONHOLDSTYLE_Both
     MinRange=512.000000
     ShakeOffsetTime=2.500000
     ErrandPickupClass=Class'Inventory.NapalmPickupErrand'
     bAllowHints=True
     bShowHints=True
     HudHint1="Press Fire(Left Mouse Button) for a straight ahead shot."
     HudHint2="Press Secondary Fire(Right Mouse Button) and be careful with this in-doors."
     FirstPersonMeshSuffix="Napalm"
     WeaponSpeedReload=0.500000
     WeaponSpeedHolster=1.500000
     WeaponSpeedShoot1=0.900000
     WeaponSpeedShoot1Rand=0.100000
     WeaponSpeedShoot2=0.500000
     WeaponSpeedShoot2Rand=0.100000
     WeaponSpeedIdle=0.400000
     AltFireSound=Sound'WeaponSounds.napalm_fire'
     AmmoName=Class'Inventory.NapalmAmmoInv'
     AutoSwitchPriority=10
     FireOffset=(X=20.000000,Y=20.000000,Z=0.000000)
     ShakeRotMag=(X=400.000000)
     ShakeRotTime=2.500000
     ShakeOffsetMag=(X=20.000000,Y=3.000000,Z=3.000000)
     TraceAccuracy=0.300000
     aimerror=300.000000
     AIRating=0.990000
     MaxRange=2048.000000
     FireSound=Sound'WeaponSounds.napalm_fire'
     InventoryGroup=10
     GroupOffset=2
     PickupClass=Class'TnInventory.NapalmLauncherPickup'
     PlayerViewOffset=(X=1.700000,Z=-1.300000)
     BobDamping=0.975000
     AttachmentClass=Class'TnInventory.NapalmLauncherAttachment'
     ItemName="Napalm launcher"
     Mesh=SkeletalMesh'TnWeapons.Tn_Dude_Napalm'
     Skins(0)=Texture'WeaponSkins.Dude_Hands'
     Skins(1)=Texture'None'
     Skins(2)=Texture'None'
     Skins(3)=Texture'ben_skins.Weapons.napalmCanister'
	 SelectSound=Sound'TnSound.Select_Heavy'
}
