///////////////////////////////////////////////////////////////////////////////
// PackageWeapon
///////////////////////////////////////////////////////////////////////////////

class PackageWeapon extends CowHeadBaseWeapon;

///////////////////////////////////////////////////////////////////////////////
// Vars, structs, consts
///////////////////////////////////////////////////////////////////////////////

var Sound BuzzingSound;
var Package TPK;

///////////////////////////////////////////////////////////////////////////////

simulated function PostNetBeginPlay()
{
    local PlayerController PC;
	local Pawn P;

    Super.PostNetBeginPlay();

    P  = Pawn(Owner);
	PC = PlayerController(P.Controller);

   	if (P.IsLocallyControlled() && PC!=None && (!PC.bBehindView) )
	{
        TPK = Spawn(Class'TnInventory.Package');
        AttachToBone(TPK, 'Package');
    }
   	else
	{
        if(Instigator != None)
	   {
		 if(Instigator.Mesh != None)

             TPK.Destroyed();
             TPK = None;
	   }
	}

}

simulated function Destroyed()
{
   Super.Destroyed();

  if( TPK != None )
  {
    DetachFromBone(TPK);
    TPK.Destroy();
    TPK = None;
  }
}


///////////////////////////////////////////////////////////////////////////////
// Play firing animation/sound/etc
// Set here that we want to reload after each throw
///////////////////////////////////////////////////////////////////////////////
simulated function PlayFiring()
{
	Super.PlayFiring();
	bForceReload=true;
}
simulated function PlayAltFiring()
{
	Super.PlayAltFiring();
	bForceReload=true;
}

///////////////////////////////////////////////////////////////////////////////
// Normal projectile fire, plus check where to make the danger marker to 
// alert people of the noise.
///////////////////////////////////////////////////////////////////////////////
function ProjectileFire()
{
	// STUB
}
///////////////////////////////////////////////////////////////////////////////
// Alt projectile fire, plus check where to make the danger marker to 
// alert people of the noise.
///////////////////////////////////////////////////////////////////////////////
function ProjectileAltFire()
{
	// STUB
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
simulated function PlayReloading()
{
	PlayAnim('Load', WeaponSpeedReload, 0.05);
}

///////////////////////////////////////////////////////////////////////////////
// Called on client's side to make the gun fire
// Check here to throw out danger markers to let people know the gun has gone
// off.
///////////////////////////////////////////////////////////////////////////////
simulated function LocalFire()
{
	local P2Player P;
	
	bPointing = true;

	// Same as the one in P2Weapon, but we don't do the camera shake here
	// We make it shake when he throws

	if ( Affector != None )
		Affector.FireEffect();
	PlayFiring();
	if(Role < ROLE_Authority)
		GotoState('ClientFiring');
}		

///////////////////////////////////////////////////////////////////////////////
// Same as above.. we don't want the shake here
///////////////////////////////////////////////////////////////////////////////
simulated function LocalAltFire()
{
	local PlayerController P;

	bPointing = true;

	// Same as the one in P2Weapon, but we don't do the camera shake here
	// We make it shake when he throws

	if ( Affector != None )
		Affector.FireEffect();
	PlayAltFiring();
	if(Role < ROLE_Authority)
		GotoState('ClientFiring');
}		

///////////////////////////////////////////////////////////////////////////////
// Actually throw the head
///////////////////////////////////////////////////////////////////////////////
function ThrowIt(float StartSpeed)
{
	local vector HitLocation, HitNormal, StartTrace, EndTrace, X,Y,Z;
	local actor HitActor;
	local PackageProjectile cowh;
	local P2Player p2p;

	if(AmmoType != None
		&& AmmoType.HasAmmo())
	{
		GetAxes(Instigator.GetViewRotation(),X,Y,Z);
		StartTrace = GetFireStart(X,Y,Z); 
		AdjustedAim = Instigator.AdjustAim(AmmoType, StartTrace, 2*AimError);	
		// Make sure we're not generating this on the other side of a thin wall
		// Also, bump anything along the way, so the grenade can break a window if
		// you're standing really close to one.
		HitActor = Trace(HitLocation, HitNormal, Instigator.Location, StartTrace, true);
		if(HitActor == None
			|| (!HitActor.bStatic
				&& !HitActor.bWorldGeometry))
		{
			cowh = spawn(class'PackageProjectile',Instigator,,StartTrace, AdjustedAim);
			if(cowh != None)
			{
				P2AmmoInv(AmmoType).UseAmmoForShot();
				cowh.Instigator = Instigator;
				cowh.CalcStartVelocity(StartSpeed);
				cowh.AddRelativeVelocity(Instigator.Velocity);

				// Shake the view when you throw it (primary fire only)
				if(!bAltFiring)
				{
					if ( Instigator != None)
					{
						p2p = P2Player(Instigator.Controller);
						if (p2p!=None)
						{
							p2p.ClientShakeView(ShakeRotMag, ShakeRotRate, ShakeRotTime, 
										ShakeOffsetMag, ShakeOffsetRate, ShakeOffsetTime);
						}
					}
				}
				// Touch any actor that was in between, just in case.
				if(HitActor != None)
					HitActor.Bump(cowh);
			}
		}
	}
	// Turn off the thing in his hand as it leaves
	if(ThirdPersonActor != None)
		ThirdPersonActor.bHidden=true;
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function Notify_ThrowCowhead()
{
	ThrowIt(1.0);
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function Notify_AltThrowCowhead()
{
	ThrowIt(0.15);
	TurnOffHint();
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Idle
//////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state Idle
{
	simulated function EndState()
	{
		Super.EndState();
		Instigator.AmbientSound=None;
	}
	///////////////////////////////////////////////////////////////////////////////
	// Make sure you're attachment is visible on idle start
	///////////////////////////////////////////////////////////////////////////////
	simulated function BeginState()
	{
		Super.BeginState();
		if(ThirdPersonActor != None)
			ThirdPersonActor.bHidden=false;
		Instigator.AmbientSound = BuzzingSound;
	}
}

///////////////////////////////////////////////////////////////////////////////
// Default properties
///////////////////////////////////////////////////////////////////////////////

defaultproperties
{
     BuzzingSound=Sound'WeaponSounds.cowhead_idle_loop'
     bUsesAltFire=True
     ViolenceRank=1
     RecognitionDist=900.000000
     ShotCountMaxForNotify=0
     holdstyle=WEAPONHOLDSTYLE_Carry
     switchstyle=WEAPONHOLDSTYLE_Carry
     firingstyle=WEAPONHOLDSTYLE_Carry
     bThrownByFiring=True
     NoAmmoChangeState="EmptyDownWeapon"
     bNoHudReticle=True
     ShakeOffsetTime=2.500000
     bAllowHints=True
     bShowHints=True
     HudHint1="Stand still and press Secondary Fire(Right Mouse Button)   "
     HudHint2="to place it safely. Then shoot it at a distance to trigger."
     bBumpStartsFight=False
     bHideFoot=True
     FirstPersonMeshSuffix="Cowhead"
     WeaponSpeedShoot1Rand=0.500000
     AmmoName=Class'TnInventory.PackageAmmoInv'
     AutoSwitchPriority=7
     FireOffset=(X=20.000000,Z=-10.000000)
     ShakeRotMag=(X=400.000000,Z=200.000000)
     ShakeRotTime=2.500000
     ShakeOffsetMag=(X=20.000000,Y=3.000000,Z=3.000000)
     TraceAccuracy=0.100000
     aimerror=500.000000
     MaxRange=1024.000000
     FireSound=Sound'WeaponSounds.cowhead_fire'
     InventoryGroup=7
     GroupOffset=3
     PickupClass=Class'TnInventory.PackagePickup'
     BobDamping=0.975000
     AttachmentClass=Class'TnInventory.PackageAttachment'
     ItemName="Package"
     Mesh=SkeletalMesh'TnWeapons.Tn_Dude_Cowhead'
     Skins(0)=Texture'WeaponSkins.Dude_Hands'
     Skins(1)=Texture'None'
}
