//=============================================================================
// M9Weapon.
//=============================================================================
class M9Weapon extends P2Weapon;

///////////////////////////////////////////////////////////////////////////////
var M9 TM9;
var bool bCanFireAgain;		// If you're the player and this is set to true, then you
							// shoot again if even you're still in NormalFire

var float WeaponSpeedShoot1MP;	// Allow faster shooting in multiplayer.
var float SPAccuracy;				// How accurate we are in single player games.

const FIRING_WAIT_TIME = 0.4;
///////////////////////////////////////////////////////////////////////////////
simulated function PostBeginPlay()
{
	local float diffoffset;

	Super.PostBeginPlay();

	// If it's a single player game, make our accuracy much worse
	if(Level.Game != None
		&& FPSGameInfo(Level.Game).bIsSinglePlayer)
	{
		TraceAccuracy = SPAccuracy;

		// Based on the difficulty, bump up the number of times AI will continuously
		// shoot this weapon
		diffoffset = P2GameInfo(Level.Game).GetDifficultyOffset();
		if(diffoffset > 0)
		{
			AI_BurstCountExtra+=(diffoffset/2);
		}
	}
}

simulated function PostNetBeginPlay()
{
    local PlayerController PC;
	local Pawn P;

    Super.PostNetBeginPlay();

    P  = Pawn(Owner);
	PC = PlayerController(P.Controller);

   	if (P.IsLocallyControlled() && PC!=None && (!PC.bBehindView) )
	{
        TM9 = Spawn(Class'TnInventory.M9');
        AttachToBone( TM9, 'M9' );
    }
   	else
	{
        if(Instigator != None)
	   {
		 if(Instigator.Mesh != None)

             TM9.Destroyed();
             TM9 = None;
	   }
	}

}

///////////////////////////////////////////////////////////////////////////////

function DropFrom(vector StartLocation)
{

   Super.DropFrom(StartLocation);

     TM9.Destroyed();

}

simulated function Destroyed()
{
   Super.Destroyed();

  if( TM9 != None )
  {
    DetachFromBone(TM9);
    TM9.Destroy();
    TM9 = None;
  }
}

///////////////////////////////////////////////////////////////////////////////
// Fire the weapon
///////////////////////////////////////////////////////////////////////////////
simulated function PlayFiring()
	{
	// Basics
	IncrementFlashCount();
	if(Level.Game == None
		|| !FPSGameInfo(Level.Game).bIsSinglePlayer)
	{
		// Play your sound in MP in your attachment to save bandwidth
		PlayAnim('Shoot1', WeaponSpeedShoot1MP + (WeaponSpeedShoot1Rand*FRand()), 0.05);
	}
	else
	{
		Instigator.PlaySound(FireSound, SLOT_None, 1.0, true, , WeaponFirePitchStart + (FRand()*WeaponFirePitchRand));
		PlayAnim('Shoot1', WeaponSpeedShoot1 + (WeaponSpeedShoot1Rand*FRand()), 0.05);
	}

	// Adv.
	SetupMuzzleFlash();
	// Once the player started his fire, don't let it fire again
	// until the sleep in the NormalFire state let's him
	if(PlayerController(Instigator.Controller) != None)
		bCanFireAgain=false;
	}

///////////////////////////////////////////////////////////////////////////////
// This will randomly change the color and the size of the dynamic
// light associate with the flash. Change in each weapon's file,
// but call each time you start up the flash again.
// This function is also used by the third-person muzzle flash, so the 
// colors will look the same
///////////////////////////////////////////////////////////////////////////////
simulated function PickLightValues()
{
	LightBrightness=150;
	LightSaturation=150;
	LightHue=12+FRand()*15;
	LightRadius=12+FRand()*6;
}

///////////////////////////////////////////////////////////////////////////////
// Make sure the muzzle flash renders, and move it around some
///////////////////////////////////////////////////////////////////////////////
simulated function SetupMuzzleFlash()
{
	bMuzzleFlash = true;
	bSetFlashTime = false;
	// Gets turned off in weapon, in RenderOverlays
	// Slightly change colors each time
	if(IsFirstPersonView()
		&& bDrawMuzzleFlash)
	{
		PickLightValues();
		bDynamicLight=bAllowDynamicLights;
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// NormalFire
// If we're in here, and it's the player and he presses fire before the fire
// anim is done and *after* the FIRING_WAIT_TIME sleep is over, then we
// let him fire again.
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
state NormalFire
{
	///////////////////////////////////////////////////////////////////////////////
	// Same as standard ServerFire
	///////////////////////////////////////////////////////////////////////////////
	function ServerFire()
	{
		if(FPSPawn(Instigator).bPlayer
			&& bCanFireAgain)
		{
			if ( AmmoType == None )
			{
				// ammocheck
				log("WARNING "$self$" HAS NO AMMO!!!");
				GiveAmmo(Pawn(Owner));
			}
			if ( AmmoType.HasAmmo() )
			{
				GotoState('NormalFire');
				if ( AmmoType.bInstantHit )
					TraceFire(TraceAccuracy,0,0);
				else
					ProjectileFire();
				LocalFire();
			}
		}
		else
			Super.ServerFire();
	}
	///////////////////////////////////////////////////////////////////////////////
	// Same as standard Fire
	///////////////////////////////////////////////////////////////////////////////
	simulated function Fire( float Value )
	{
		if(FPSPawn(Instigator).bPlayer
			&& bCanFireAgain)
		{
			if ( AmmoType == None
				|| !AmmoType.HasAmmo() )
			{
			ClientForceFinish();
			ServerForceFinish();
				return;
			}

			if ( !RepeatFire() )
				ServerFire();
			else if ( StopFiringTime < Level.TimeSeconds + 0.3 )
			{
				StopFiringTime = Level.TimeSeconds + 0.6;
				ServerRapidFire();
			}
			if ( Role < ROLE_Authority )
			{
				bCanFireAgain=false;
				LocalFire();
				GotoState('ClientFiring');
			}
		}
		else
			Super.Fire(Value);
	}

	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	simulated function EndState()
	{
		bCanFireAgain=true;
		Super.EndState();
	}
Begin:
	Sleep(FIRING_WAIT_TIME);
	bCanFireAgain=true;
}
state ClientFiring
{
}

///////////////////////////////////////////////////////////////////////////////
// Default properties
///////////////////////////////////////////////////////////////////////////////

defaultproperties
{
	 SelectSound=Sound'TnSound.Select_M9'
	 bCanFireAgain=True
     SPAccuracy=0.150000
     ViolenceRank=3
     RecognitionDist=900.000000
     AI_BurstCountExtra=3
     AI_BurstTime=0.800000
     ShotMarkerMade=Class'Postal2Game.GunfireMarker'
     BulletHitMarkerMade=Class'Postal2Game.BulletHitMarker'
     holdstyle=WEAPONHOLDSTYLE_Single
     switchstyle=WEAPONHOLDSTYLE_Single
     firingstyle=WEAPONHOLDSTYLE_Single
     MinRange=250.000000
     ShakeOffsetTime=2.200000
     CombatRating=3.000000
     FirstPersonMeshSuffix="Pistol"
     WeaponSpeedLoad=1.500000
     WeaponSpeedReload=1.500000
     WeaponSpeedHolster=1.500000
     WeaponSpeedShoot1Rand=0.040000
     AmmoName=Class'TnInventory.M9AmmoInv'
     AutoSwitchPriority=2
     ShakeRotMag=(X=220.000000,Y=30.000000,Z=30.000000)
     ShakeRotTime=2.200000
     ShakeOffsetMag=(X=10.000000,Y=2.000000,Z=2.000000)
     TraceAccuracy=0.020000
     aimerror=400.000000
     AIRating=0.200000
     MaxRange=1024.000000
     FireSound=Sound'TnSound.Shot_M9'
     FlashOffsetY=0.150000
     FlashOffsetX=0.215000
     FlashLength=0.010000
     MuzzleFlashSize=128.000000
     MFTexture=Texture'Timb.muzzle_flash.pistol_corona'
     bDrawMuzzleFlash=True
     MuzzleFlashScale=2.400000
     MuzzleFlashStyle=STY_Normal
     InventoryGroup=2
     GroupOffset=2
     PickupClass=Class'TnInventory.M9Pickup'
     PlayerViewOffset=(X=0.000000,Z=-1.000000)
     BobDamping=0.975000
     AttachmentClass=Class'TnInventory.M9Attachment'
     ItemName="Beretta M9"
     Mesh=SkeletalMesh'TnWeapons.Tn_Dude_Pistol'
     Skins(0)=Texture'WeaponSkins.Dude_Hands'
     Skins(1)=Texture'None'
     Skins(2)=Texture'None'
     AmbientGlow=64
     SoundRadius=255.000000
}
