///////////////////////////////////////////////////////////////////////////////
// TnGasCanWeapon
///////////////////////////////////////////////////////////////////////////////

class TnGasCanWeapon extends P2WeaponStreaming;

#exec OBJ LOAD FILE=TnMesh.usx
#exec OBJ LOAD FILE=TnWeapons.ukx

///////////////////////////////////////////////////////////////////////////////
var GasPourFeeder gaspour;
var bool	bPerformingAltFire;
var Sound	soundMatchStrike;
var() Rotator FireRotator;
var TnGasCan TGC;

const MATCH_LAUNCH_VEL_MAG	=	800;
///////////////////////////////////////////////////////////////////////////////

simulated function PostNetBeginPlay()
{
    local PlayerController PC;
	local Pawn P;

    Super.PostNetBeginPlay();

    P  = Pawn(Owner);
	PC = PlayerController(P.Controller);

   	if (P.IsLocallyControlled() && PC!=None && (!PC.bBehindView) )
	{
        TGC = Spawn(Class'TnInventory.TnGasCan');
        AttachToBone(TGC, 'TnGasCan');
    }
   	else
	{
        if(Instigator != None)
	   {
		 if(Instigator.Mesh != None)

             TGC.Destroyed();
             TGC = None;
	   }
	}

}

simulated function Destroyed()
{
   Super.Destroyed();

  if( TGC != None )
  {
    DetachFromBone(TGC);
    TGC.Destroy();
    TGC = None;
  }
}

///////////////////////////////////////////////////////////////////////////////
// Play animation/sound/etc
///////////////////////////////////////////////////////////////////////////////
simulated function PlayAltFiring()
	{
	// This "throw the match" animation must include a notification that
	// call Notify_LightMatch() in order for the match to actually work!
	PlayAnim('Shoot2', WeaponSpeedShoot2, 0.05);
	}

///////////////////////////////////////////////////////////////////////////////
// we don't shoot things
///////////////////////////////////////////////////////////////////////////////
function TraceFire( float Accuracy, float YOffset, float ZOffset )
	{
	// STUB
	}

///////////////////////////////////////////////////////////////////////////////
// Make the gas pour go to the end of the gun and orient itself
///////////////////////////////////////////////////////////////////////////////
function SnapGasPourToGun(optional bool bInitArc)
	{
	local vector startpos, X,Y,Z;
	local vector forward;
	local Rotator myrot;
	
	// orient gas pour and reposition
	myrot = Instigator.GetViewRotation();
	myrot.Pitch += FireRotator.Pitch;
	myrot.Pitch = myrot.Pitch & 65535;
	myrot.Yaw += FireRotator.Yaw;
	myrot.Yaw = myrot.Yaw & 65535;
	myrot.Roll += FireRotator.Roll;
	myrot.Roll = myrot.Roll & 65535;
	
	GetAxes(myrot,X,Y,Z);
	forward = Normal(vector(myrot));
	startpos = GetFireStart(X,Y,Z);// + forward*4;
	gaspour.SetLocation(startpos);
	gaspour.SetDir(startpos, vector(myrot),,bInitArc);
	}

///////////////////////////////////////////////////////////////////////////////
// Generate a match
///////////////////////////////////////////////////////////////////////////////
simulated function GenerateMatch()
	{
	local vector HitLocation, HitNormal, StartTrace, EndTrace, X,Y,Z;
	local actor Other;
	local FireMatch thematch;
	
	GetAxes(Instigator.GetViewRotation(),X,Y,Z);
	StartTrace = GetFireStart(X,Y,Z); 
	AdjustedAim = Instigator.AdjustAim(AmmoType, StartTrace, 2*AimError);	
	thematch = spawn(class'FireMatch',Instigator,,StartTrace, AdjustedAim);
	if(thematch != None)
		thematch.Velocity = MATCH_LAUNCH_VEL_MAG*Normal(vector(thematch.Rotation));
	}

///////////////////////////////////////////////////////////////////////////////
function ServerAltFire()
	{
	bPerformingAltFire=true;
	
	GotoState('AltFireState');
	LocalAltFire();
	}

///////////////////////////////////////////////////////////////////////////////
function ServerFire()
	{
	bPerformingAltFire=false;
	
	Super.ServerFire();
	}

///////////////////////////////////////////////////////////////////////////////
// Terminate the stream
///////////////////////////////////////////////////////////////////////////////
simulated function bool ForceEndFire()
	{
	if(gaspour != None)
		{
		gaspour.ToggleFlow(0.0, false);
		gaspour = None;
		return Super.ForceEndFire();
		}
	return false;
	}

///////////////////////////////////////////////////////////////////////////////
// Animation notification called when the correct point is reached in the
// "light the match" animation.
///////////////////////////////////////////////////////////////////////////////
simulated function Notify_LightMatch()
	{
	// Generates this in anim
	//Instigator.PlaySound(soundMatchStrike, SLOT_None, 1.0);
	GenerateMatch();
	TurnOffHint();
	}
	
///////////////////////////////////////////////////////////////////////////////
// Streaming, make gas come out in this mode
///////////////////////////////////////////////////////////////////////////////
state Streaming
{
	function Tick( float DeltaTime )
	{
		// Cut immediately if you stop early
		if(!Instigator.PressingFire())
			GotoState('EndStream');
		else
		{
			SnapGasPourToGun();
			
			ReduceAmmo(DeltaTime);
		}
	}

	function EndState()
	{
		Super.EndState();
		ForceEndFire();
	}

	function BeginState()
	{
		Super.BeginState();
		gaspour = spawn(class'GasPourFeeder',Instigator,,,Rotation);
		gaspour.MyOwner = Owner;
		SnapGasPourToGun(true);
	}
}

///////////////////////////////////////////////////////////////////////////////
state Idle
	{
	function BeginState()
		{

		bPerformingAltFire=false;
		
		// If instigator doesn't want to fire anymore then we can finally
		// end the whole pouring sequence.
		if (!Instigator.PressingFire())
			{
			if (Owner != None)
				ForceEndFire();
			}
		}
	}

///////////////////////////////////////////////////////////////////////////////
state AltFireState extends NormalFire
{
	function Tick( float DeltaTime )
		{
		if(!bPerformingAltFire)
			SnapGasPourToGun();
		}
}


///////////////////////////////////////////////////////////////////////////////
// Default properties
///////////////////////////////////////////////////////////////////////////////

defaultproperties
{
     soundMatchStrike=Sound'WeaponSounds.match_light'
     FireRotator=(Yaw=-1500)
     soundStart=Sound'WeaponSounds.GasCan_Start'
     soundLoop1=Sound'WeaponSounds.GasCan_Loop'
     soundLoop2=Sound'WeaponSounds.GasCan_Loop'
     soundEnd=Sound'WeaponSounds.GasCan_End'
     bUsesAltFire=True
     ViolenceRank=1
     RecognitionDist=900.000000
     holdstyle=WEAPONHOLDSTYLE_Pour
     switchstyle=WEAPONHOLDSTYLE_Carry
     firingstyle=WEAPONHOLDSTYLE_Pour
     bNoHudReticle=True
     ShakeOffsetTime=2.000000
     PlayerMeleeDist=200.000000
     NPCMeleeDist=200.000000
     bAllowHints=True
     bShowHints=True
     HudHint1="Secondary Fire(Right Mouse Button) tosses a match."
     bBumpStartsFight=False
     FirstPersonMeshSuffix="GasCan"
     WeaponSpeedLoad=1.500000
     WeaponSpeedReload=1.500000
     WeaponSpeedHolster=1.500000
     WeaponSpeedShoot1Rand=0.200000
     AmmoName=Class'TnInventory.TnGasCanBulletAmmoInv'
     AutoSwitchPriority=5
     FireOffset=(X=40.000000,Y=10.000000)
     TraceAccuracy=0.900000
     bMeleeWeapon=True
     aimerror=0.000000
     AIRating=0.150000
     InventoryGroup=5
     GroupOffset=3
     PickupClass=Class'TnInventory.TnGasCanPickup'
     PlayerViewOffset=(Z=-1.000000)
     BobDamping=0.975000
     AttachmentClass=Class'TnInventory.TnGasCanAttachment'
     ItemName="Gas Can"
     Mesh=SkeletalMesh'TnWeapons.Tn_Dude_GasCan'
	 Skins(0)=Texture'WeaponSkins.Dude_Hands'
     Skins(1)=Texture'None'
     Skins(2)=Texture'None'
}
